
import keyboard as k
from time import sleep
from platform import python_version, system, python_version_tuple

# import functions
from .functions.cls import *
from .functions.print_header import *
from .functions.clear_buffer import *
from .functions.debugprint import *

# import classes
from .classes.theme import *
from .classes.exceptions import *
from .classes.menu_options import *

try:
    import msvcrt
except:
    raise menuException(f'failed to import "msvcrt" module in {__file__}')

version = '0.2.28'

__all__=[
    'menu',
    'version',
    'selection_delay',
    'refresh_rate'
]

re_iterations = 0
choice = 1
last_page = ''
last_teme = ''
last_theme = ''


selection_delay = 0.22  # 0.22 seems best
refresh_rate = 0.01     # 0.01 seems appropriate

# menu master class
class menu(object):

    def __init__(self, program_name):
        self.name = program_name

    def generate_selection(self, page_name, menu_items:list, menu_text:list = None, theme:str = None):

        '''
        generates an in-terminal user-selection menu that works both in windows command-prompt and powershell.

        '''

        # self setup
        #theme = theme
        #program_name = program_name
        #page_name = page_name
        #menu_items = menu_items
        #if menu_text != None:
        #    menu_text = menu_text

        global choice     
        global last_page
        global last_theme
        global re_iterations

        clear_buffer()

        # this ensures that when changing pages, the selection returns to the top of the page
        if last_page != '':
            if last_page != page_name:
                choice = 1
        else:
            choice = 1
        last_page = page_name

        # only set the theme when applicable
        if last_theme != theme:
            if theme != None:
                color_theme.set_theme(theme)             
        last_theme = theme

        # get the total amount of menu_options loaded into the menu, minus any "_skip_" options.
        # if the choice goes above or below the number of loaded menu items, return to 1 or total.
        # this ensures that the options scroll in the desired way.
        total_options = 0
        total_options = len(menu_items) - menu_items.count('_skip_')
        if choice > total_options:
            choice = 1
        if choice < 1:
            choice = total_options


        # clear terminal screen and print menu header
        cls()
        print_header(self.name, page_name)

        # prints optional menu_text, if there is any.
        if menu_text != None:
            if isinstance(menu_text, str) == True:
                print(f' {color_theme.text}{menu_text}{color_theme.end}')
            elif isinstance(menu_text, list) == True:
                for item in menu_text:
                    if item == '_skip_':
                        print('')
                        continue
                    print(f' {color_theme.text}{item}{color_theme.end}')
            else:
                raise menuException('menu_text must either be a string or a list')
            print('')

        # if no menu options are present, raise exception (can't have a menu with no options, can you?).
        if len(menu_items) <= 0:
            raise Exception('menu must contain at least one menu item')

        # print menu items
        i = 1
        if isinstance(menu_items, str) == True:
            menu_option(i, item, True).print()
        elif isinstance(menu_items, list) == True:
            for item in menu_items:
                if item == '_skip_':
                    print('')
                    continue
                if i == choice:
                    menu_option(i, item, True).print()
                else:
                    menu_option(i, item).print()
                i += 1
        else:
            menuException('menu_items must either be a string or a list')

        # l00p until user selects an item in the menu
        sleep(selection_delay)
        while not k.is_pressed('enter'):
            if k.is_pressed('s'):
                try:
                    choice += 1
                    return self.generate_selection(page_name, menu_items, menu_text, theme)
                except:
                    return
            elif k.is_pressed('w'):
                try:
                    choice -= 1
                    return self.generate_selection(page_name, menu_items, menu_text, theme)
                except:
                    self.terminate()
            sleep(refresh_rate)
        
        return menu_option(choice, menu_items[choice-1], menu_items[choice-1]).select()

    def terminate(self):
        clear_buffer()
        del self
        return

#check that user is using the correct OS and version of python
if system().lower() != 'windows':
    raise menuException(f'invalid operating system: "{system()}"\nyetimenu currently only works on windows machines.')
if int(python_version_tuple()[0]) < 3 or int(python_version_tuple()[1]) < 11:
    raise menuException(f'invalid python version: "{python_version()}"\nyetimenu makes use of features, such as f-strings, that require python 3.6 or newer.')
debugprint('boop', 5)




from time import sleep
from configparser import ConfigParser
from .exceptions import *

__all__=[
    'color_theme',
    'formatting'
]

class color_theme:

    # default color theme (white)
    text = '\33[37m'
    highlight = '\33[7m'
    error = '\033[91m'
    title = '\33[7m'
    page = '\33[37m'
    end = '\033[0m'

    def set_theme(color:str):
        if color.lower() in ['blue', 'bleu']:

            # colors
            color_theme.text = '\33[34m'
            color_theme.highlight = '\33[44m\033[30m'
            color_theme.error = '\033[91m'
            color_theme.title = '\33[44m\033[30m'
            color_theme.page = '\33[34m'
        else: #sets the default theme
            pass

        return

class formatting:

    title_sep = '::'
    page_brackets = ['[', ']']

    def set_formatting(title_sep:str, page_brackets:list):

        if title_sep == None:
            formatting.title_sep == ''
        elif isinstance(title_sep, str):
            formatting.title_sep = title_sep
        else:
            raise menuException('title_sep must be a string')
        

        if page_brackets == None:
            formatting.page_brackets = ''
        elif isinstance(page_brackets, list):
            if len(page_brackets) != 2:
                raise menuException(f'page_brackets MUST be a list containing exactly 2 items, your list contains {len(page_brackets)} item(s)')
            else:
                formatting.page_brackets = page_brackets
        else:
            raise menuException(f'page_brackets MUST be a list containing exactly 2 items. You entered a {type(page_brackets)}')



from menu.classes.exceptions import *

try:
    from os import system, name
except:
    raise menuException(f'failed to import "os" module in {__file__}')

__all__=[
    'cls'
]

# function that clears the terminal screen when called
def cls():
    system('cls' if name=='nt' else 'clear')
    return


from .theme import *

class menuprint(object):

    '''
    A print() function that prints in color
    
    '''

    def __init__(self, msg:str, error:bool = False, newline:bool = True):
        self.msg = msg
        if error != False:
            if newline == True:
                print(f'\n {color_theme.error}*{self.msg}{color_theme.end}')
            else:
                print(f' {color_theme.error}*{self.msg}{color_theme.end}')
        else:
            if newline == True:
                print(f'\n {color_theme.text}{self.msg}{color_theme.end}')
            else:
                print(f' {color_theme.text}{self.msg}{color_theme.end}')


from menu.classes.exceptions import *

try:
    from msvcrt import kbhit, getch
except:
    raise menuException(f'failed to import "msvcrt" module in {__file__}')

__all__=[
    'clear_buffer'
]

def clear_buffer():

    # clears the input buffer
    try:
        while kbhit():
            getch()
    except:
        raise menuException(f'failed to clear input buffer in file: {__file__}')
    
    return


import menu as m
from time import sleep

app_name = 'yetimenu: example menu'
my_theme = ''

def primary_menu():

    global my_theme

    if my_theme == '':
        my_theme = 'Purple'
    menu_type = 'selection'
    page_name = 'main menu'
    menu_items = [
        'module information',               #CASE 1
        'change theme via selection',       #CASE 2
        'change theme via input',           #CASE 3
        '_skip_',                           # _skip_ IS IGNORED BY MENU
        '[X] exit'                          #CASE 4
    ]

    selection = m.generate(menu_type, my_theme, app_name, page_name, menu_items)

    match selection:
        case 1:
            return module_menu()
        case 2:
            return theme_selection_menu()
        case 3:
            return print('bar'), sleep(2)
        case 4:
            m._exit_()

        case _: #wildcard
            return print('error'), exit()

def module_menu():

    page_name = 'module information'
    menu_type = 'selection'
    text = [
        'name: yetimenu',
        'author: deadyeti (deadyeti@deadyeti.ca)',
        'created: 2021-OCT-12',
        'last updated: 2021-OCT-13',
        '_skip_',
        '--- description ---',
        '"a simple python module to quickly and easily generate in-terminal menus"'
    ]
    menu_items = [
        '[<] back',                #CASE 1
        '[X] exit'                 #CASE 2
    ]

    selection = m.generate(menu_type, my_theme, app_name, page_name, menu_items, text)

    match selection:
        case 1:
            return primary_menu()
        case 2:
            m._exit_()

        case _: #wildcard
            return print('wut?'), exit()

def theme_selection_menu():

    global my_theme

    page_name = 'theme selection'
    menu_type = 'selection'
    text = [
        'please select from the available built-in themes below:'
    ]

    menu_items = [
        'Red Theme',                 #CASE 1
        'Green Theme',               #CASE 2
        'Blue Theme',                #CASE 3
        'Light Blue Theme',          #CASE 4
        'Yellow Theme',              #CASE 5
        'White Theme',               #CASE 6
        '_skip_',                    # _skip_ IS IGNORED BY MENU
        '[<] back',                  # CASE 7
        '[X] exit'                   #CASE 8
    ]

    selection = m.generate(menu_type, my_theme, app_name, page_name, menu_items, text)

    match selection:
            case 1:
                my_theme = 'Red'
                return theme_selection_menu()
            case 2:
                my_theme = 'Green'
                return theme_selection_menu()
            case 3:
                my_theme = 'Blue'
                return theme_selection_menu()
            case 4:
                my_theme = 'Light Blue'
                return theme_selection_menu()
            case 5:
                my_theme = 'Yellow'
                return theme_selection_menu()
            case 6:
                my_theme = 'White'
                return theme_selection_menu()
            case 7:
                return
            case 8:
                m._exit_()

            case _: #wildcard
                return print('error'), exit()

while True:
    primary_menu()



from menu.classes.theme import *
from menu.classes.exceptions import *
from .clear_buffer import *


__all__=[
    'menuinput'
]

def menuinput(msg:str, newline:bool = False):

    '''
    Exactly like input() except it uses the same color as the menu color theme.
    '''

    # clear input buffer, important before any input
    clear_buffer()
    
    if newline == False:
        user_input = input(f' {color_theme.text}{msg}') + f'{color_theme.end}'
    else:
        user_input = input(f' {color_theme.text}{msg}') + f'{color_theme.end}'
    return user_input




"""
menu.py
=============================================
A simple python package for in-terminal menu creation on Windows

copyright (c) 2021-present deadyeti

MIT License, see LICENSE for more details.
"""

# import functions
#from .menu import generate
#from .menu import terminate

# import classes
#from .menu import cprint
#from .menu import eprint

# import modules
from .menu import *
from .functions.menuinput import *
from .functions.menuprint import *
from .classes.theme import formatting

__all__ = [
    'menu',
    'menuprint',
    'menuprint_error',
    'menuinput',
    'formatting'
]

__title__ = 'menu'
__author__ = 'deadyeti'
__license__ = 'MIT'
__copyright__ = 'Copyright 2021-present deadyeti'
__version__ = version


from .theme import *

__all__=[
    'menu_option'
]

# menu_option subclass
class menu_option(object):

    def __init__(self, number:int, name:str, bHighlight:bool = False):
        self.number = number
        self.name = name
        self.bHighlight = bHighlight

    def print(self):
        if self.bHighlight == False:
            print(f' {color_theme.text} {self.name} {color_theme.end}')
        else:
            print(f' {color_theme.highlight} {self.name} {color_theme.end}')
        return

    def select(self):
        return self.number